﻿:Class  Latest2
⍝ User Command script for Kai's "Lastest"
⍝ Kai Jaeger
⍝ Version 1.1.9 from 2017-02-24
⍝ 1.1.8  : Workaround for the CommandFolder/WinReg key/Separator problem implemented.
⍝ 1.1.18 : Documentation imporoved.
⍝ 1.1.17 : minor improvements to the documentation.

    ∇ r←List;⎕IO;⎕ML ⍝ this function usually returns 1 or more namespaces (here only 1)
      :Access Shared Public
      ⎕IO←⎕ML←1
      r←⎕NS''                                         ⍝ create command
      r.Name←'Latest2'               ⍝ the name
      r.Desc←'Prints all changed functions and operators sorted by "Changed" date'
      r.Group←'APLTree'                                      ⍝
     ⍝ Parsing rules for each:
      r.Parse←' -s= -a -d'            ⍝ takes one optional argument and three optional switches
    ∇

    ∇ r←Run(Cmd Args);source;all;noOf;ref;asDay
      :Access Shared Public
      ⎕IO←⎕ML←1
      ref←CopyCode
      source←''Args.Switch's'   ⍝ default is empty
      all←0 Args.Switch'a'
      asDay←0 Args.Switch'd'
      :If 0∊⍴Args.Arguments
          noOf←⍬
      :Else
          noOf←⍎1⊃Args.Arguments
      :EndIf
      :If 0∊⍴source
          r←ref.Latest.List({⍵≡⍬:⍵ ⋄ (1+asDay)⊃1 ¯1×⍵}noOf)(~all)
      :Else
          r←source ref.Latest.List({⍵≡⍬:⍵ ⋄ (1+asDay)⊃1 ¯1×⍵}noOf)(~all)
      :EndIf
    ∇

    ∇ r←Help Cmd;⎕IO;⎕ML;sep
      ⎕IO←⎕ML←1
      sep←⎕UCS 13
      :Access Shared Public
      r←''
      r,←'Returns a list with timestamp and author for ...',sep
      r,←'* all fns/opr in the workspace when the argument is empty or undefined.',sep
      r,←'* the latest y fns/opr when y (the right agrument) is a positive integer.',sep
      r,←'* all fns/opr changed during the last |y days when y is negative.',sep
      r,←'',sep
      r,←'-s= : with the optional switch -s= one can specify a certain namespace to look into.',sep
      r,←'-a  : list all.',sep
      r,←'-d  : interpret the numeric value a number of days (rather than number of items),',sep
      r,←'Note that fns/opr from scripted namespaces will be ignored.',sep
    ∇

    ∇ r←ReadUtf8File filename;fno;noOfBytes;bytes;⎕IO;⎕ML
      ⎕IO←⎕ML←1
      r←''
      :Trap 22
          fno←filename ⎕NTIE 0
      :Else
          ('Could not read file: ',filename)⎕SIGNAL ⎕EN
      :EndTrap
      noOfBytes←⎕NSIZE fno
      bytes←⎕NREAD fno 83,noOfBytes
      ⎕NUNTIE fno
      ⍝ Make sure it is unsigned, and drop the UTF-8 marker bytes if present
      bytes+←256×bytes<0
      bytes↓⍨←3×239 187 191≡3↑bytes ⍝ drop a potential UTF-8 marker
      r←'UTF-8'⎕UCS bytes
      :If ∨/(⎕UCS 13 10)⍷r
          r←Split r
      :ElseIf ∨/r=⎕UCS 10
          r←(⎕UCS 10)Split r
      :EndIf
    ∇

      Split←{
          ⎕ML←⎕IO←1
          ⍺←⎕UCS 13 10 ⍝ Default is CR+LF
          (⍴,⍺)↓¨⍺{⍵⊂⍨⍺⍷⍵}⍺,⍵
      }

    ∇ ref←CheckRefs ref
      :If '#'≠1⍴⍕ref
          ref←'#.',⍕ref
      :EndIf
      :If 3≠⎕NC ref
          ref←⍎ref
      :EndIf
    ∇

    ∇ ref←CopyCode;regKey;paths;success;thisPath;filename;regData
      regKey←GetRegKey,'\SALT\CommandFolder'
      regData←ReadRegKey regKey
      ((regData∊'∘°')/regData)←';'
      paths←';'Split regData
      success←0
      :For thisPath :In paths
          filename←thisPath,'\Latest2.DWS'
          ref←⎕NS''
          :Trap 0
              ref.⎕CY filename
              :If 0<1↑⍴ref.⎕NL⍳16
                  success←1
                  :Leave
              :EndIf
          :EndTrap
      :EndFor
      'Cannot find Latest.dws'⎕SIGNAL 11/⍨~success
    ∇

    ∇ r←{default}ReadRegKey key;wsh;⎕WX
     ⍝ Read a registry key.
      ⎕WX←1
      default←{0<⎕NC ⍵:⍎⍵ ⋄ ''}'default'
      'wsh'⎕WC'OLEClient' 'WScript.Shell'
      :Trap 11
          r←wsh.RegRead key
      :Else
          r←default
      :EndTrap
    ∇

    ∇ regKey←GetRegKey;⎕IO;⎕ML
      ⎕IO←⎕ML←1
      regKey←''
      regKey,←'HKEY_CURRENT_USER\Software\Dyalog\Dyalog APL/W'
      regKey,←('-64'≡¯3↑1⊃'.'⎕WG'APLVersion')/'-64'
      regKey,←' ',{⍵/⍨1≥+\'.'=⍵}2⊃'.'⎕WG'APLVersion'
      regKey,←' ',((80=⎕DR'')/'Unicode')
    ∇

:EndClass